#include <stdio.h>
#include <pcap.h>
#include <sys/time.h>
#include <signal.h>
#include <string.h>

#include "headers.h"

int unpack_it(unsigned char *pack, u_int32_t psize, u_int32_t *info, u_int16_t off, u_int16_t size);

u_int32_t packet_receiver(struct header_hash_table **head_hash, u_int16_t noh, struct recv *recv);

pcap_t *init_pcap(char *device, char *filter_opts, int *lnk_off);
