int in_cksum(unsigned short *addr, int len);

u_int32_t tcp_cksum(char *ipp, char *tcpp, int len);

u_int16_t do_tcp_checksum(u_char *pack,\
		          unsigned char zero,\
			  unsigned char proto,\
			  unsigned short len,\
			  u_char *ip_addr);

u_int16_t do_icmp_checksum(u_char *pack,int icmpsize);
